#ifndef CoCoA_SmallFpLogImpl_H
#define CoCoA_SmallFpLogImpl_H

//   Copyright (c)  2005,2009,2011,2012  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


// Header file for the class SmallFpLogImpl


#include "CoCoA/assert.H"
#include "CoCoA/config.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/MachineInt.H"

#include <vector>
// using std::vector


namespace CoCoA
{

  class BigInt;         // fwd decl -- defined in BigInt.H
  class BigRat;         // fwd decl -- defined in BigRat.H
  class OpenMathOutput; // fwd decl -- defined in OpenMath.H

  class SmallFpLogImpl
  {
  public:
    explicit SmallFpLogImpl(long p, GlobalSettings::ResidueSetting ResidueChoice = DefaultResidueSetting());
    static bool IsGoodCtorArg(long p);
//    SmallFpLogImpl(const BigInt& P);
  private:
    SmallFpLogImpl(const SmallFpLogImpl&);            // NEVER DEFINED -- copy ctor disabled
    SmallFpLogImpl& operator=(const SmallFpLogImpl&); // NEVER DEFINED -- assignment disabled

  public:
    typedef SmallFpLogElem_t value_t;
    static const int ourDatumSize = sizeof(value_t);

    long myModulus() const;
    value_t myReduce(const MachineInt& n) const;                  ///< n % myModulus
    value_t myReduce(const BigInt& N) const;                      ///< N % myModulus
    value_t myReduce(const BigRat& q) const;                      ///< q % myModulus (error if den(q)%myModulus == 0)
    value_t mySemiReduce(value_t n) const;
//     void myAssignZero(value_t& lhs) const;                        ///< lhs = 0
//     void myAssign(value_t& lhs, value_t x) const;                 ///< lhs = x
//     void myAssign(value_t& lhs, const MachineInt& n) const;   ///< lhs = n
//     void myAssign(value_t& lhs, const BigInt& N) const;           ///< lhs = N
//     void myAssign(value_t& lhs, const BigRat& q) const;           ///< lhs = q
//     void myNegate(value_t& lhs, value_t x) const;                 ///< lhs = -x
//     void myAdd(value_t& lhs, value_t x, value_t y) const;         ///< lhs = x+y
//     void mySub(value_t& lhs, value_t x, value_t y) const;         ///< lhs = x-y
//     void myMul(value_t& lhs, value_t x, value_t y) const;         ///< lhs = x*y
//     void myDiv(value_t& lhs, value_t x, value_t y) const;         ///< lhs = x/y
///    void myNegate(value_t& lhs, value_t x) const;                 ///< lhs = -x
    value_t myNegate(value_t x) const;
///    void myAdd(value_t& lhs, value_t x, value_t y) const;         ///< lhs = x+y
    value_t myAdd(value_t x, value_t y) const;
///    void mySub(value_t& lhs, value_t x, value_t y) const;         ///< lhs = x-y
    value_t mySub(value_t x, value_t y) const;
///    void myMul(value_t& lhs, value_t x, value_t y) const;         ///< lhs = x*y
    value_t myMul(value_t x, value_t y) const;
///    void myDiv(value_t& lhs, value_t x, value_t y) const;         ///< lhs = x/y
    value_t myDiv(value_t x, value_t y) const;
    bool myIsDivisible(value_t& lhs, value_t x, value_t y) const; ///< lhs = x/y, if y is non-zero
///    void myPower(value_t& lhs, value_t x, long n) const;          ///< lhs = x^n
///    void myPower(value_t& lhs, value_t x, long n) const;          ///< lhs = x^n
    value_t myPower(value_t x, long n) const;
///    value_t myReduceMod(const MachineInt& n) const;           ///< computes n % myModulus

//     bool myIsZero(value_t x) const;                        ///< x == 0
//     bool myIsOne(value_t x) const;                         ///< x == 1
//     bool myIsMinusOne(value_t x) const;                    ///< x == -1
    long myExport(value_t x) const;                        ///< exports x into a long (according to myResiduesAreSymm)
    bool myIsZeroAddMul(value_t& lhs, value_t y, value_t z) const;// lhs += y*z, result says whether lhs == 0.
//    bool myIsEqual(value_t x, value_t y) const;  // x == y

  private: // data members
    const value_t myModulusValue;
    const bool myResiduesAreSymm;   // Used only in myExport
    const value_t myDrop;     // PUBLIC???
    const value_t myIterLimit;// PUBLIC???
    const value_t myRoot;
    typedef unsigned short FpTableElem; // the log/exp table elements are of this type
    std::vector<FpTableElem> myLog;
    std::vector<FpTableElem> myExp;

  private:
    static value_t CheckCtorArg(long p);
//    static value_t CheckCtorArg(const BigInt& P);
    static value_t CalcDrop(value_t p);
    static value_t CalcIterLimit(value_t p);
    void myCtorBody();
  };  // end of class SmallFpLogImpl


  std::ostream& operator<<(std::ostream& out, const SmallFpLogImpl& arith);


  //------------------------------------------------------------
  // SmallFpLogImpl inline functions
  //------------------------------------------------------------

  inline long SmallFpLogImpl::myModulus() const
  {
    return myModulusValue; // implicit cast is safe
  }


  inline bool operator==(const SmallFpLogImpl& arith1, const SmallFpLogImpl& arith2)
  {
    return (arith1.myModulus() == arith2.myModulus());
  }


  inline SmallFpLogImpl::value_t SmallFpLogImpl::mySemiReduce(value_t n) const
  {
    if (n < myDrop) return n;
    return n-myDrop;
  }


//   inline void SmallFpLogImpl::myNegate(value_t& lhs, value_t x) const
//   {
//     CoCoA_ASSERT(x == x%myModulusValue);
//     if (x == 0) lhs = x;
//     else lhs = myModulusValue-x;
//   }

  inline SmallFpLogImpl::value_t SmallFpLogImpl::myNegate(value_t x) const
  {
    CoCoA_ASSERT(x == myReduce(x));
    if (x == 0) return x;
    return myModulusValue - x;
  }



//   inline void SmallFpLogImpl::myAdd(value_t& lhs, value_t x, value_t y) const
//   {
//     CoCoA_ASSERT(x == x%myModulusValue);
//     CoCoA_ASSERT(y == y%myModulusValue);
//     lhs = x+y;
//     if (lhs >= myModulusValue) lhs -= myModulusValue;
//   }

  inline SmallFpLogImpl::value_t SmallFpLogImpl::myAdd(value_t x, value_t y) const
  {
    CoCoA_ASSERT(x == myReduce(x));
    CoCoA_ASSERT(y == myReduce(y));
    value_t ans = x+y;
    if (ans < myModulusValue) return ans;
    return ans - myModulusValue;
  }


//   inline void SmallFpLogImpl::mySub(value_t& lhs, value_t x, value_t y) const
//   {
//     CoCoA_ASSERT(x == x%myModulusValue);
//     CoCoA_ASSERT(y == y%myModulusValue);
//     if (x < y) lhs = x + (myModulusValue-y);  // avoid trying to create a negative value
//     else lhs = x-y;
//   }


  inline SmallFpLogImpl::value_t SmallFpLogImpl::mySub(value_t x, value_t y) const
  {
    CoCoA_ASSERT(x == myReduce(x));
    CoCoA_ASSERT(y == myReduce(y));
    if (x >= y) return x-y;
    return x + (myModulusValue-y);  // avoid trying to create a negative value
  }

//   inline void SmallFpLogImpl::myMul(value_t& lhs, value_t x, value_t y) const
//   {
//     CoCoA_ASSERT(x == x%myModulusValue);
//     CoCoA_ASSERT(y == y%myModulusValue);
//     if (x == 0 || y == 0) lhs = 0;
//     else lhs = myExp[myLog[x] + myLog[y]];
//   }


  inline SmallFpLogImpl::value_t SmallFpLogImpl::myMul(value_t x, value_t y) const
  {
    CoCoA_ASSERT(x == myReduce(x));
    CoCoA_ASSERT(y == myReduce(y));
    if (x == 0 || y == 0) return 0;
    else return myExp[myLog[x] + myLog[y]];
  }

//   inline void SmallFpLogImpl::myDiv(value_t& lhs, value_t x, value_t y) const
//   {
//     CoCoA_ASSERT(x == x%myModulusValue);
//     CoCoA_ASSERT(y == y%myModulusValue);
//     CoCoA_ASSERT(!myIsZero(y));
//     if (x == 0) lhs = 0;
//     else lhs = myExp[myModulusValue-1 + myLog[x] - myLog[y]];
//   }

  inline SmallFpLogImpl::value_t SmallFpLogImpl::myDiv(value_t x, value_t y) const
  {
    CoCoA_ASSERT(x == myReduce(x));
    CoCoA_ASSERT(y == myReduce(y));
    CoCoA_ASSERT(y != 0);
    if (x == 0) return 0;
    return myExp[myModulusValue-1 + myLog[x] - myLog[y]];
  }


  inline bool SmallFpLogImpl::myIsDivisible(value_t& lhs, value_t x, value_t y) const
  {
    CoCoA_ASSERT(x == myReduce(x));
    CoCoA_ASSERT(y == myReduce(y));
    if (y == 0) return false;
    lhs = myDiv(x, y);
    return true;
  }



  inline long SmallFpLogImpl::myExport(value_t x) const
  {
    if (!myResiduesAreSymm || x <= myModulusValue/2) return x; // implicit cast is safe
    return -long(myModulusValue - x); // implicit cast is safe
  }


  inline bool SmallFpLogImpl::myIsZeroAddMul(value_t& lhs, value_t y, value_t z) const
  {
    CoCoA_ASSERT(y == myReduce(y));
    CoCoA_ASSERT(z == myReduce(z));
    CoCoA_ASSERT(lhs == myReduce(lhs));
    lhs += myMul(y,z); if (lhs >= myModulusValue) lhs -= myModulusValue;
    return (lhs == 0);
  }


} // end of namespace CoCoA


// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/SmallFpLogImpl.H,v 1.13 2012/09/07 15:21:13 abbott Exp $
// $Log: SmallFpLogImpl.H,v $
// Revision 1.13  2012/09/07 15:21:13  abbott
// First stage of revision of SmallFpImpl interface (and SmallFpLog, SmallFpDouble).
//
// Revision 1.12  2012/01/30 11:01:11  abbott
// Added naive printing fn (just to help during debugging).
//
// Revision 1.11  2011/11/09 13:50:01  bigatti
// -- renamed MachineInteger --> MachineInt
//
// Revision 1.10  2011/08/24 10:21:09  bigatti
// -- renamed QQ --> BigRat
//
// Revision 1.9  2011/08/14 15:52:17  abbott
// Changed ZZ into BigInt (phase 1: just the library sources).
//
// Revision 1.8  2011/05/20 19:26:05  abbott
// Updated SmallFp*Impl: removed all output-related fns (must use myExport instead).
//
// Revision 1.7  2011/05/19 14:38:27  abbott
// Updated small prime finite field impls to allow user to specify
// separately for each whether to use symmetric or non-negative
// residues for export operations (myExport and printing).
//
// Revision 1.6  2011/03/22 20:06:13  abbott
// Added static mem fn IsGoodCtorArg (called by RingFp pseudo-ctors).
// Commented out ctors which take ZZ arg -- seems useless.
//
// Revision 1.5  2011/03/14 10:28:15  abbott
// Changed unsigned long into long (and unsigned short into short).
//
// Revision 1.4  2009/05/14 09:39:29  abbott
// Added possibility to specify "symmetric" or "non-negative" residues
// in quotients of ZZ.  Affects printing of elements in quotients of ZZ
// (also changed printing of elements in general quotient rings).
// Consequent changes in several tests.
//
// Revision 1.3  2008/12/17 12:11:52  abbott
// Changed type from long to MachineInt in operations which use a machine integer
// in place of a RingElem.  The change is "superficial" but affects many files.
//
// Revision 1.2  2007/10/30 17:14:11  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.7  2007/03/08 18:42:05  cocoa
// Cleaned up whitespace.
//
// Revision 1.6  2007/01/11 14:07:42  cocoa
// -- changed names to arguments called "rsh"
//
// Revision 1.5  2006/12/06 17:30:02  cocoa
// -- rearranged #include
//
// Revision 1.4  2006/11/24 17:22:05  cocoa
// -- removed OpenMathFwd.H
//
// Revision 1.3  2006/11/02 13:25:44  cocoa
// Simplification of header files: the OpenMath classes have been renamed.
// Many minor consequential changes.
//
// Revision 1.2  2006/10/06 14:04:15  cocoa
// Corrected position of #ifndef in header files.
// Separated CoCoA_ASSERT into assert.H from config.H;
// many minor consequential changes (have to #include assert.H).
// A little tidying of #include directives (esp. in Max's code).
//
// Revision 1.1.1.1  2006/05/30 11:39:37  cocoa
// Imported files
//
// Revision 1.5  2006/05/12 16:10:58  cocoa
// Added OpenMathFwd.H, and tidied OpenMath.H.
// Many consequential but trivial changes.
//
// Revision 1.4  2006/03/27 12:21:26  cocoa
// Minor silly changes to reduce number of complaints from some compiler or other.
//
// Revision 1.3  2006/03/21 09:43:14  cocoa
// Changed names of some member fns of ideals (dealing with setting and testing
// the flags for primeness and maximality).  Hope icc will complain less now.
//
// Revision 1.2  2006/03/12 21:28:34  cocoa
// Major check in after many changes
//
// Revision 1.1.1.1  2005/10/17 10:46:54  cocoa
// Imported files
//
// Revision 1.5  2005/10/14 15:25:07  cocoa
// Major tidying and cleaning to small prime finite fields.
// Several consequential changes.  Improved their documentation.
//
// Added Makefile and script to include/CoCoA/ directory to
// keep library.H up to date.
//
// Revision 1.4  2005/10/12 15:52:09  cocoa
// Completed test-RingFp1 and corrected/cleaned the SmallFp*
// and RingFp* files.
//
// Some minor tidying elsewhere.
//
// Revision 1.3  2005/10/11 16:37:30  cocoa
// Added new small prime finite field class (see RingFpDouble).
//
// Cleaned makefiles and configuration script.
//
// Tidied PPMonoid code (to eliminate compiler warnings).
//
// Fixed bug in RingFloat::myIsInteger.
//
// Revision 1.2  2005/09/22 18:04:17  cocoa
// It compiles; the tests run OK.  The examples compile.
// No documentation -- the mindless eurocrats have rendered
// me mindless too.
//
// Revision 1.1.1.1  2005/05/03 15:47:31  cocoa
// Imported files
//
// Revision 1.4  2005/04/19 14:06:04  cocoa
// Added GPL and GFDL licence stuff.
//
// Revision 1.3  2005/02/28 15:58:56  cocoa
// Resynch after some minor changes.
//
// Revision 1.2  2005/02/11 16:45:24  cocoa
// Removed the useless and misleading functions myInit and myKill
// from the SmallFp*Impl classes; various consequential changes.
//
// Revision 1.1.1.1  2005/01/27 15:12:13  cocoa
// Imported files
//
// Revision 1.4  2004/11/12 15:49:29  cocoa
// Tidying prior to 0.90 release.
// (a) documentation improved (or marked as poor)
// (b) sundry minor improvements to the code
//
// Revision 1.3  2004/11/04 18:47:42  cocoa
// (1) Ring member functions which previously expected mpz_t args
//     now expect ZZ args.  Numerous minor consequential changes.
// (2) Renamed function which gives access to the mpz_t value inside
//     a ZZ object: previously was raw(...), now is mpzref(...).
//     Plenty of calls had to be altered.
//
// Revision 1.2  2004/07/16 15:45:12  cocoa
// First stage of new RingElem implementation completed.
//
// Revision 1.1  2004/07/14 16:40:42  cocoa
// Separated RingFpLog from its implementation which now resides in
// a new class: SmallFpLogImpl.  This is analogous to the change made
// to RingFp yesterday.
//
// Some tidying and other sundry minor changes.
//
//


#endif
